R3FRAME_QUOTES: list[str] = [
    "howdy gamers!",
    "its all about perspective...",
    "now with colors!",
    "simple interfaces, complex tasks...",
    "made with <3",
    "what will you create?",
    "release date? when?",
    "is it... stable yet?",
    "the beauty is in the design!"
]